import logging
from typing import Protocol, runtime_checkable, Any, Union
from .simple_file_logger import SimpleFileLogger
from .logger_with_id import LogResult as _LogResult
from dataclasses import dataclass
from uuid import uuid4


@runtime_checkable
class Bot(Protocol):
    async def send_message(self, chat_id: int, message: str, parse_mode: str):
        raise NotImplementedError


@dataclass
class LogResult(_LogResult):
    message: Union[Any, None]
    message_id: Union[int, None]


class BotLogger(SimpleFileLogger):
    def __init__(self, name, bot: Bot, chat_id: int, level=logging.DEBUG, filename=..., hidden_url='https://google.com'):
        super().__init__(name, level, filename)

        if not isinstance(bot, Bot):
            raise NotImplementedError('bot object is not valid instance of Bot')

        self.bot = bot
        self.chat_id = chat_id
        self.parse_mode = 'Markdown'
        self.hidden_url = hidden_url
        self.limit = 4000

    async def send_event(self, msg: str, chat_id: int = ..., **kwargs) -> Any:
        if chat_id is ...:
            chat_id = self.chat_id

        return await self.bot.send_message(chat_id, msg, parse_mode=self.parse_mode, **kwargs)

    async def info(self, msg, stream_only=False, chat_id: int = ..., *args, **kwargs):
        return await self.log(logging.INFO, msg, stream_only, chat_id, *args, **kwargs)

    async def warning(self, msg, stream_only=False, chat_id: int = ..., *args, **kwargs):
        return await self.log(logging.WARNING, msg, stream_only, chat_id, *args, **kwargs)

    async def error(self, msg, stream_only=False, chat_id: int = ..., *args, **kwargs):
        return await self.log(logging.ERROR, msg, stream_only, chat_id, *args, **kwargs)

    async def critical(self, msg, stream_only=False, chat_id: int = ..., *args, **kwargs):
        return await self.log(logging.CRITICAL, msg, stream_only, chat_id, *args, **kwargs)

    async def log(self, level, msg, stream_only=False, chat_id: int = ..., *args, **kwargs):
        unique_id = uuid4()
        magic_url = f"{self.hidden_url}/{unique_id}"
        message = None
        message_id = None

        if not stream_only:
            bot_msg = msg[:self.limit] + '...' if len(msg) > self.limit else msg
            bot_msg = f'{bot_msg}[.]({magic_url})'
            message = await self.send_event(bot_msg, chat_id, **kwargs)
            message_id = message.id if hasattr(message, 'id') else message.message_id

        log_msg = f"{msg} [MESSAGE_ID: {message_id}]"
        self._log(level, log_msg, args, unique_id=unique_id, **kwargs)

        return LogResult(unique_id, log_msg, message, message_id)



import logging
from typing import Any
from uuid import uuid4, UUID
from dataclasses import dataclass

@dataclass
class LogResult:
    id: UUID
    msg: str

class LoggerWithID(logging.Logger):
    def __init__(self, name, level=logging.NOTSET):
        super().__init__(name, level=level)

    def debug(self, msg, *args, **kwargs):
        return self._log(logging.DEBUG, msg, args, **kwargs)

    def info(self, msg, *args, **kwargs):
        return self._log(logging.INFO, msg, args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        return self._log(logging.WARNING, msg, args, **kwargs)

    def error(self, msg, *args, **kwargs):
        return self._log(logging.ERROR, msg, args, **kwargs)

    def critical(self, msg, *args, **kwargs):
        return self._log(logging.CRITICAL, msg, args, **kwargs)

    def _log(self, level, msg, args, unique_id=None, **kwargs):
        if not unique_id:
            unique_id = uuid4()

        msg = f"{msg} [LOG_ID: {unique_id}]"

        if self.isEnabledFor(level):
            super()._log(level, msg, args, **kwargs)

        return LogResult(id=unique_id, msg=msg)



__all__ = [
    'LoggerWithID',
    'SimpleFileLogger',
    'QualNameLogger',
    'BotLogger',
]

from .bot_logger import BotLogger
from .logger_with_id import LoggerWithID
from .simple_file_logger import SimpleFileLogger
from .qualname_logger import QualNameLogger



from .async_repository import AbstractAsyncRepository
from .async_uow import AbstractUOW, AbstractUOWFactory, BaseRepoCollector, TRepo

__all__ = [
    "AbstractAsyncRepository",
    "BaseRepoCollector",
    "TRepo",
    "AbstractUOW",
    "AbstractUOWFactory",
]



import os
import logging
import inspect


class QualNameLogger(logging.Logger):
    def __init__(self, name: str, level = 0, base_path=...) -> None:
        super().__init__(name, level)

        if base_path is ...:
            frame = self._get_init_frame()
            base_path = os.path.dirname(frame.f_code.co_filename) if frame else None

        self.base_path = base_path

    def _get_init_frame(self):
        frame = inspect.currentframe()

        while frame and 'self' in frame.f_locals and frame.f_locals['self'] is self:
            frame = frame.f_back

        return frame

    def _get_entity(self, frame):
        entity = frame.f_globals.get(frame.f_code.co_name)
        return entity

    def get_qualname(self):
        frame = self._get_init_frame()

        if not frame:
            return '__main__'

        entity = self._get_entity(frame)

        filename = frame.f_code.co_filename
        if self.base_path:
            filename = os.path.relpath(filename, self.base_path)

        if not entity:
            return f"{filename}"

        qualname = entity.__qualname__

        return f"{filename}:{qualname}:{frame.f_code.co_firstlineno}"

    def _log(self, level, msg, args, **kwargs):
        qualname = self.get_qualname()
        msg = f'({qualname}): {msg}'

        return super()._log(level, msg, args, **kwargs)



from setuptools import find_packages, setup

with open("VERSION", "r") as f:
    VERSION = f.read().strip()

DESCIPTION = "library that defines abstractions for some repeated essences"

with open("requirements.txt", "r") as requirements_file:
    REQUIREMENTS = requirements_file.readlines()


setup(
    name="axabc",
    version=VERSION,
    author="axdjuraev",
    author_email="<axdjuraev@gmail.com>",
    description=DESCIPTION,
    packages=find_packages(),
    install_requires=REQUIREMENTS,
)
